@@ -2,6 +2,7 @@ use std::env;
 use std::fs;
 use std::path::{Path, PathBuf};
 
+#[derive(Debug, PartialEq, Eq)]
 pub enum Version {
     Lua51,
     Lua52,
@@ -55,11 +56,11 @@ impl Build {
         let include_dir = out_dir.join("include");
 
         let source_dir_base = Path::new(env!("CARGO_MANIFEST_DIR"));
-        let source_dir = match version {
+        let mut source_dir = match version {
             Lua51 => source_dir_base.join("lua-5.1.5"),
             Lua52 => source_dir_base.join("lua-5.2.4"),
             Lua53 => source_dir_base.join("lua-5.3.6"),
-            Lua54 => source_dir_base.join("lua-5.4.4"),
+            Lua54 => source_dir_base.join("lua-5.4.6"),
         };
 
         if lib_dir.exists() {
@@ -84,13 +85,7 @@ impl Build {
             _ if target.contains("linux") => {
                 config.define("LUA_USE_LINUX", None);
             }
-            _ if target.contains("freebsd") => {
-                config.define("LUA_USE_LINUX", None);
-            }
-            _ if target.contains("netbsd") => {
-                config.define("LUA_USE_LINUX", None);
-            }
-            _ if target.contains("openbsd") => {
+            _ if target.ends_with("bsd") => {
                 config.define("LUA_USE_LINUX", None);
             }
             _ if target.contains("apple-darwin") => {
@@ -99,10 +94,44 @@ impl Build {
                     _ => config.define("LUA_USE_MACOSX", None),
                 };
             }
+            _ if target.contains("apple-ios") => {
+                match version {
+                    Lua54 => config.define("LUA_USE_IOS", None),
+                    _ => config.define("LUA_USE_POSIX", None),
+                };
+            }
             _ if target.contains("windows") => {
                 // Defined in Lua >= 5.3
                 config.define("LUA_USE_WINDOWS", None);
             }
+            _ if target.ends_with("emscripten") => {
+                config
+                    .define("LUA_USE_POSIX", None)
+                    .cpp(true)
+                    .flag("-fexceptions"); // Enable exceptions to be caught
+
+                let cpp_source_dir = out_dir.join("cpp_source");
+                if cpp_source_dir.exists() {
+                    fs::remove_dir_all(&cpp_source_dir).unwrap();
+                }
+                fs::create_dir_all(&cpp_source_dir).unwrap();
+
+                for file in fs::read_dir(&source_dir).unwrap() {
+                    let file = file.unwrap();
+                    let filename = file.file_name();
+                    let filename = filename.to_str().unwrap();
+                    let src_file = source_dir.join(file.file_name());
+                    let dst_file = cpp_source_dir.join(file.file_name());
+
+                    let mut content = fs::read(src_file).unwrap();
+                    if ["lauxlib.h", "lua.h", "lualib.h"].contains(&filename) {
+                        content.splice(0..0, b"extern \"C\" {\n".to_vec());
+                        content.extend(b"\n}".to_vec())
+                    }
+                    fs::write(dst_file, content).unwrap();
+                }
+                source_dir = cpp_source_dir
+            }
             _ => panic!("don't know how to build Lua for {}", target),
         };
 
